macro_rules! impl_add {
($Lhs:ty) => { ... };
}Expand description
implement the Add, Sub, and Mul traits for NodeRef and FSNodeRef
macro_rules! impl_add {
($Lhs:ty) => { ... };
}implement the Add, Sub, and Mul traits for NodeRef and FSNodeRef