CircuitBuilder

Struct CircuitBuilder 

pub struct CircuitBuilder<F: Field> {
Show 13 fields input_layer_nodes: Vec<Rc<InputLayerNode>>, input_shred_nodes: Vec<Rc<InputShred>>, fiat_shamir_challenge_nodes: Vec<Rc<FiatShamirChallengeNode>>, output_nodes: Vec<Rc<OutputNode>>, sector_nodes: Vec<Rc<Sector<F>>>, gate_nodes: Vec<Rc<GateNode>>, identity_gate_nodes: Vec<Rc<IdentityGateNode>>, split_nodes: Vec<Rc<SplitNode>>, matmult_nodes: Vec<Rc<MatMultNode>>, lookup_constraint_nodes: Vec<Rc<LookupConstraint>>, lookup_table_nodes: Vec<Rc<LookupTable>>, node_to_ptr: HashMap<NodeId, NodeRef<F>>, circuit_map: CircuitMap,
}
A struct that owns and manages super::nodes::CircuitNodes during circuit creation.

Fields

input_layer_nodes: Vec<Rc<InputLayerNode>>input_shred_nodes: Vec<Rc<InputShred>>fiat_shamir_challenge_nodes: Vec<Rc<FiatShamirChallengeNode>>output_nodes: Vec<Rc<OutputNode>>sector_nodes: Vec<Rc<Sector<F>>>gate_nodes: Vec<Rc<GateNode>>identity_gate_nodes: Vec<Rc<IdentityGateNode>>split_nodes: Vec<Rc<SplitNode>>matmult_nodes: Vec<Rc<MatMultNode>>lookup_constraint_nodes: Vec<Rc<LookupConstraint>>lookup_table_nodes: Vec<Rc<LookupTable>>node_to_ptr: HashMap<NodeId, NodeRef<F>>circuit_map: CircuitMap

Implementations

Source

impl<F: Field> CircuitBuilder<F>

pub fn new() -> Self

Constructs an empty CircuitBuilder.

fn into_owned_helper<T: CircuitNode + Debug>(xs: Vec<Rc<T>>) -> Vec<T>

pub fn build_with_max_layer_size( self, maybe_maximum_log_layer_size: Option<usize>, ) -> Result<Circuit<F>>

Generates a circuit description of all the nodes added so far.

Returns a Circuit struct containing the circuit description and all necessary metadata for attaching inputs.

pub fn build_with_layer_combination(self) -> Result<Circuit<F>>

A build function that combines layers greedily such that the circuit is optimized for having the smallest number of layers possible.

pub fn build_without_layer_combination(self) -> Result<Circuit<F>>

A build function that does not combine any layers.

pub fn build(self) -> Result<Circuit<F>>

A default build function which does not combine layers. Equivalent to build_without_layer_combination.

Source

impl<F: Field> CircuitBuilder<F>

pub fn add_input_layer( &mut self, layer_label: &str, layer_visibility: LayerVisibility, ) -> InputLayerNodeRef<F>

Adds an InputLayerNode labeled layer_label to the builder’s node collection, intented to become a layer_kind input later during circuit instantiation.

Returns a weak pointer to the newly created layer node.

Note that Input Layers and Input Shred have disjoint label scopes. A label has to be unique only in its respective scope, regardless of the inclusive relation between shreds and input layers.

Panics

If layer_label has already been used for an existing Input Layer.

pub fn add_input_shred( &mut self, label: &str, num_vars: usize, source: &InputLayerNodeRef<F>, ) -> NodeRef<F>

Returns a reference to the newly created node.

Note that no method in Self requires to differentiate between a reference to an input shred as opposed to a generic NodeRef, so there is no need to retain the specific type information in the returned type.

Panics

If label has already been used for an existing Input Shred.

pub fn set_output(&mut self, source: &NodeRef<F>)

Adds an zero OutputNode (using OutputNode::new_zero()) to the builder’s node collection.

TODO(Makis): Add a check for ensuring each node can be set as output at most once.

pub fn add_fiat_shamir_challenge_node( &mut self, num_challenges: usize, ) -> FSNodeRef<F>

Adds a FiatShamirChallengeNode to the builder’s node colllection. Returns a typed reference to the newly created node.

pub fn add_sector(&mut self, expr: AbstractExpression<F>) -> NodeRef<F>

Adds a Sector to the builder’s node collection. Returns a typed reference to the newly created node.

pub fn add_identity_gate_node( &mut self, pre_routed_data: &NodeRef<F>, non_zero_gates: Vec<(u32, u32)>, num_vars: usize, num_dataparallel_vars: Option<usize>, ) -> NodeRef<F>

Adds an IdentityGateNode to the builder’s node collection. Returns a reference to the newly created node.

Note that no method in Self requires to differentiate between a reference to an identity gate node as opposed to a generic NodeRef, so there is no need to retain the specific type information in the returned type.

pub fn add_gate_node( &mut self, lhs: &NodeRef<F>, rhs: &NodeRef<F>, nonzero_gates: Vec<(u32, u32, u32)>, gate_operation: BinaryOperation, num_dataparallel_bits: Option<usize>, ) -> NodeRef<F>

Adds an GateNode to the builder’s node collection. Returns a reference to the newly created node.

Note that no method in Self requires to differentiate between a reference to a gate node as opposed to a generic NodeRef, so there is no need to retain the specific type information in the returned type.

pub fn add_matmult_node( &mut self, matrix_a_node: &NodeRef<F>, rows_cols_num_vars_a: (usize, usize), matrix_b_node: &NodeRef<F>, rows_cols_num_vars_b: (usize, usize), ) -> NodeRef<F>

Adds an MatMultNode to the builder’s node collection. Returns a reference to the newly created node.

Note that no method in Self requires to differentiate between a reference to a matmult node as opposed to a generic NodeRef, so there is no need to retain the specific type information in the returned type.

pub fn add_lookup_table( &mut self, table: &NodeRef<F>, fiat_shamir_challenge_node: &FSNodeRef<F>, ) -> LookupTableNodeRef

Adds an LookupTable to the builder’s node collection. Returns a typed reference to the newly created node.

pub fn add_lookup_constraint( &mut self, lookup_table: &LookupTableNodeRef, constrained: &NodeRef<F>, multiplicities: &NodeRef<F>, ) -> LookupConstraintNodeRef

Adds an LookupConstraint to the builder’s node collection. Returns a typed reference to the newly created node.

pub fn add_split_node( &mut self, input_node: &NodeRef<F>, num_vars: usize, ) -> Vec<NodeRef<F>>

Adds an SplitNode to the builder’s node collection. Returns a vector of reference to the 2^num_vars newly created nodes.

Note that no method in Self requires to differentiate between a reference to a split node as opposed to a generic NodeRef, so there is no need to retain the specific type information in the returned type.

fn get_ptr_from_node_id(&self, id: NodeId) -> Rc<dyn CircuitNode>