pub struct CircuitMap {
state: CircuitMapState,
shreds_in_layer: HashMap<LayerId, Vec<NodeId>>,
label_to_shred_id: HashMap<String, NodeId>,
layer_label_to_layer_id: HashMap<String, LayerId>,
layer_visibility: HashMap<LayerId, LayerVisibility>,
node_location: HashMap<NodeId, (CircuitLocation, usize)>,
}Expand description
Manages the relations between all different kinds of identifiers used to specify nodes during circuit building and circuit instantiation. Keeps track of [LayerId]s, NodeIds, Labels, LayerVisibilitys, [CircuitLocation]s.
Fields
state: CircuitMapStateshreds_in_layer: HashMap<LayerId, Vec<NodeId>>label_to_shred_id: HashMap<String, NodeId>layer_label_to_layer_id: HashMap<String, LayerId>layer_visibility: HashMap<LayerId, LayerVisibility>node_location: HashMap<NodeId, (CircuitLocation, usize)>Implementations
Sourceimpl CircuitMap
impl CircuitMap
Sourcepub fn new() -> Self
pub fn new() -> Self
Constructs an empty CircuitMap in a CircuitMapState::UnderConstruction state.
It can receive new data until the Self::freeze method is called which will transition it
to the CircuitMapState::Ready state, at which point it can only be used to answer queries.
Sourcepub fn add_node_id_and_location_num_vars(
&mut self,
node_id: NodeId,
value: (CircuitLocation, usize),
)
pub fn add_node_id_and_location_num_vars( &mut self, node_id: NodeId, value: (CircuitLocation, usize), )
Sourcepub fn insert_shreds_into_input_layer(
&mut self,
input_layer_id: LayerId,
shreds: Vec<NodeId>,
)
pub fn insert_shreds_into_input_layer( &mut self, input_layer_id: LayerId, shreds: Vec<NodeId>, )
Sourcepub fn get_location_num_vars_from_node_id(
&self,
node_id: &NodeId,
) -> Result<&(CircuitLocation, usize)>
pub fn get_location_num_vars_from_node_id( &self, node_id: &NodeId, ) -> Result<&(CircuitLocation, usize)>
Using node_id, retrieves the number of variables and location of this
node in the circuit, or returns an error if node_id is missing.
This method is safe to use in any CircuitMapState.
Sourcepub fn add_input_layer(
&mut self,
layer_id: LayerId,
layer_label: &str,
layer_kind: LayerVisibility,
)
pub fn add_input_layer( &mut self, layer_id: LayerId, layer_label: &str, layer_kind: LayerVisibility, )
Sourcepub fn add_input_shred(&mut self, label: &str, shred_id: NodeId)
pub fn add_input_shred(&mut self, label: &str, shred_id: NodeId)
Adds a new Input Shred labeled label with node ID shred_id.
Panics
If self is not in a CircuitMapState::UnderConstruction, or if label is already in
use.
Note
While this method does not panic if shred_id has already been given a different label,
it is considered a semantic error to associate two different labels with the same node, and
Self::freeze may detect it and panic .
Sourcepub fn freeze(self) -> Self
pub fn freeze(self) -> Self
Desctructively mutates self to transition it from a CircuitMapState::UnderConstruction
state to a CircuitMapState::Ready state. As part of the transition, consistency checks are
performed to ensure all internal mappings have all the expected properties.
Panics
If self is already in a CircuitMapState::Ready state, or if its internal state is
inconsistent.
Sourcepub fn get_node_kind(&self, shred_label: &str) -> Result<LayerVisibility>
pub fn get_node_kind(&self, shred_label: &str) -> Result<LayerVisibility>
Returns the LayerVisibility of the Input Layer that the shred with label shred_label is in,
or an error if the shred_label does not correspond to any input shred.
Sourcepub fn get_node_id(&self, shred_label: &str) -> Result<NodeId>
pub fn get_node_id(&self, shred_label: &str) -> Result<NodeId>
Returns the NodeId of the Input Shred labeled shred_label, or an error if shred_label
does not correspond to any input shred.
Sourcepub fn get_all_input_shred_ids(&self) -> Vec<NodeId>
pub fn get_all_input_shred_ids(&self) -> Vec<NodeId>
Sourcepub fn get_shred_label_from_id(&self, shred_id: NodeId) -> Result<String>
pub fn get_shred_label_from_id(&self, shred_id: NodeId) -> Result<String>
Sourcepub fn get_all_input_layer_ids(&self) -> Vec<LayerId>
pub fn get_all_input_layer_ids(&self) -> Vec<LayerId>
Sourcepub fn get_all_public_input_layer_ids(&self) -> Vec<LayerId>
pub fn get_all_public_input_layer_ids(&self) -> Vec<LayerId>
Sourcepub fn get_input_shreds_from_layer_id(
&self,
layer_id: LayerId,
) -> Result<Vec<NodeId>>
pub fn get_input_shreds_from_layer_id( &self, layer_id: LayerId, ) -> Result<Vec<NodeId>>
Sourcepub fn get_shred_location(
&self,
shred_id: NodeId,
) -> Result<(CircuitLocation, usize)>
pub fn get_shred_location( &self, shred_id: NodeId, ) -> Result<(CircuitLocation, usize)>
Sourcepub fn get_all_committed_layers(&self) -> Vec<LayerId>
pub fn get_all_committed_layers(&self) -> Vec<LayerId>
Returns a vector with all [LayerId]s of the Input Layers with LayerVisibility::Committed visibility.
Panics
If self is not in CircuitMapState::Ready state.
Sourcepub fn get_layer_id_from_label(&self, layer_label: &str) -> Result<LayerId>
pub fn get_layer_id_from_label(&self, layer_label: &str) -> Result<LayerId>
Returns the layer ID of the input layer with label layer_label, or error if no such layer
exists.