pub struct Circuit<F: Field> {
circuit_description: GKRCircuitDescription<F>,
pub circuit_map: CircuitMap,
partial_inputs: HashMap<NodeId, MultilinearExtension<F>>,
}Expand description
A circuit whose structure is fixed, but is not yet ready to be proven or verified because its missing all or some of its input data. This structs provides an API for attaching inputs and generating a form of the circuit that can be proven or verified respectively, for various proving systems (vanilla GKR with Ligero, or Hyrax).
Fields
circuit_description: GKRCircuitDescription<F>circuit_map: CircuitMappartial_inputs: HashMap<NodeId, MultilinearExtension<F>>Implementations
Sourceimpl<F: Field> Circuit<F>
impl<F: Field> Circuit<F>
Sourcefn new(
circuit_description: GKRCircuitDescription<F>,
circuit_map: CircuitMap,
) -> Self
fn new( circuit_description: GKRCircuitDescription<F>, circuit_map: CircuitMap, ) -> Self
Constructor to be used by CircuitBuilder.
Sourcepub fn get_circuit_description(&self) -> &GKRCircuitDescription<F>
pub fn get_circuit_description(&self) -> &GKRCircuitDescription<F>
Return the [GKRCircuitDescription] inside this Circuit.
Sourcepub fn set_input(&mut self, shred_label: &str, data: MultilinearExtension<F>)
pub fn set_input(&mut self, shred_label: &str, data: MultilinearExtension<F>)
Assign data to the Input Shred with label shred_label.
Panics
If shred_label does not correspond to any Input Shred, or if the this Input Shred has
already been assigned data. Use Self::update_input for replacing the data of a shred.
Sourcepub fn update_input(&mut self, shred_label: &str, data: MultilinearExtension<F>)
pub fn update_input(&mut self, shred_label: &str, data: MultilinearExtension<F>)
Assign data to the Input Shred with label shred_label, discarding any existing data
associated with this Input Shred.
Panics
If shred_label does not correspond to any Input Shred.
Sourcepub fn contains_layer(&self, label: &str) -> bool
pub fn contains_layer(&self, label: &str) -> bool
Returns whether the circuit contains an Input Layer labeled label.
fn input_shred_contains_data(&self, shred_id: NodeId) -> bool
Sourcepub fn input_layer_contains_data(&self, label: &str) -> Result<bool>
pub fn input_layer_contains_data(&self, label: &str) -> Result<bool>
Returns whether data has already been assigned to the Input Layer labeled label, or an
error if no such input layer exists.
Sourcepub fn get_input_layer_description_ref(
&self,
layer_label: &str,
) -> &InputLayerDescription
pub fn get_input_layer_description_ref( &self, layer_label: &str, ) -> &InputLayerDescription
Returns the Input Layer Description of the Input Layer with label layer_label.
Panics
If no such layer exists, or if self is in an inconsistent state.
Sourcefn build_input_layer_data(
&self,
layer_id: LayerId,
) -> Result<MultilinearExtension<F>>
fn build_input_layer_data( &self, layer_id: LayerId, ) -> Result<MultilinearExtension<F>>
Builds the layer MLE for layer_id by combining the data in all the input shreds of that layer.
Returns error if layer_id is an invalid input layer ID, or if any shred data is missing.
fn build_public_input_layer_data( &self, verifier_optional_inputs: bool, ) -> Result<HashMap<LayerId, MultilinearExtension<F>>>
fn build_all_input_layer_data( &self, ) -> Result<HashMap<LayerId, MultilinearExtension<F>>>
Sourcefn get_all_committed_input_layer_descriptions_to_ligero(
&self,
) -> Vec<LigeroInputLayerDescription<F>>
fn get_all_committed_input_layer_descriptions_to_ligero( &self, ) -> Vec<LigeroInputLayerDescription<F>>
Helper function for grabbing all of the committed input layers + descriptions from the circuit map.
We do this by first filtering all input layers which are LayerVisibility::Committed, getting all input “shreds” which correspond to those input layers, and aggregating those to compute the number of variables required to represent each input layer.
Finally, we set a default configuration for the Ligero PCS used to commit to each of the committed input layers’ MLEs. TODO(tfHARD team): add support for custom settings for the PCS configurations.
Sourcepub fn gen_hyrax_verifiable_circuit<C>(
&self,
) -> Result<HyraxVerifiableCircuit<C>>where
C: PrimeOrderCurve<Scalar = F>,
pub fn gen_hyrax_verifiable_circuit<C>(
&self,
) -> Result<HyraxVerifiableCircuit<C>>where
C: PrimeOrderCurve<Scalar = F>,
Returns a [HyraxVerifiableCircuit] containing the public input layer data that have been
added to self so far.
Sourcepub fn gen_hyrax_provable_circuit<C>(&self) -> Result<HyraxProvableCircuit<C>>where
C: PrimeOrderCurve<Scalar = F>,
pub fn gen_hyrax_provable_circuit<C>(&self) -> Result<HyraxProvableCircuit<C>>where
C: PrimeOrderCurve<Scalar = F>,
Produces a provable form of this circuit for the Hyrax-GKR proving system which uses Hyrax
as a commitment scheme for private input layers, and offers zero-knowledge guarantees.
Requires all input data to be populated (use Self::set_input() on all input shreds).
Returns
The generated provable circuit, or an error if the self is missing input data.
Sourceimpl<F: Halo2FFTFriendlyField> Circuit<F>
impl<F: Halo2FFTFriendlyField> Circuit<F>
Sourcepub fn gen_provable_circuit(&self) -> Result<ProvableCircuit<F>>
pub fn gen_provable_circuit(&self) -> Result<ProvableCircuit<F>>
Produces a provable form of this circuit for the vanilla GKR proving system which uses
Ligero as a commitment scheme for committed input layers, and does not offer any
zero-knowledge guarantees.
Requires all input data to be populated (use Self::set_input() on all input shreds).
Returns
The generated provable circuit, or an error if the self is missing input data.
Sourcepub fn gen_verifiable_circuit(&self) -> Result<VerifiableCircuit<F>>
pub fn gen_verifiable_circuit(&self) -> Result<VerifiableCircuit<F>>
Returns a [VerifiableCircuit] initialized with all input data which is already known to the verifier, but no commitments to the data in the committed input layers yet.